@@ -183,8 +183,6 @@ void init_thumbnail_mode(void)
 		winwidget_show(winwid);
 	}
 
-	/* make sure we have an ~/.thumbnails/normal directory for storing
-	   permanent thumbnails */
 	td.cache_thumbnails = opt.cache_thumbnails;
 
 	if (td.cache_thumbnails) {
@@ -193,9 +191,15 @@ void init_thumbnail_mode(void)
 		else
 			td.cache_dim = opt.thumb_h;
 
-		if (td.cache_dim > 256) {
-			/* No caching as specified by standard. Sort of. */
+		if (td.cache_dim > 1024) {
+			/* Not specified by XDG thumbnail standard */
 			td.cache_thumbnails = 0;
+		} else if (td.cache_dim > 512) {
+			td.cache_dim = 1024;
+			td.cache_dir = estrdup("xx-large");
+		} else if (td.cache_dim > 256) {
+			td.cache_dim = 512;
+			td.cache_dir = estrdup("x-large");
 		} else if (td.cache_dim > 128) {
 			td.cache_dim = 256;
 			td.cache_dir = estrdup("large");
@@ -699,6 +703,13 @@ int feh_thumbnail_generate(Imlib_Image * image, feh_file * file,
 				thumb_h = td.cache_dim / ratio;
 			else if (ratio != 1.0)
 				thumb_w = td.cache_dim * ratio;
+		} else {
+			/*
+			 * The image is smaller than the specified thumbnail size.
+			 * Do not cache or transform it.
+			 */
+			*image = im_temp;
+			return 1;
 		}
 
 		*image = gib_imlib_create_cropped_scaled_image(im_temp, 0, 0, w, h,
